@@ -1,16 +1,8 @@
 // use claim::debug_assert_ok;
-use std::iter::Zip;
-use std::ops::*;
 
-use crate::data::line_segment::*;
 use crate::data::Cursor;
 use crate::data::DirectedEdge;
 use crate::data::Point;
-use crate::data::PointId;
-use crate::data::Polygon;
-use crate::data::Position;
-use crate::data::PositionId;
-use crate::data::RingId;
 
 pub struct Iter<'a, T: 'a> {
   pub(crate) iter: std::slice::Iter<'a, Point<T, 2>>,
