@@ -13,9 +13,9 @@ use std::ops::Mul;
 use std::ops::Neg;
 use std::ops::Sub;
 
-use crate::array::*;
 use crate::data::Point;
 use crate::Extended;
+use crate::Orientation;
 
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
 #[repr(transparent)]
@@ -112,7 +112,7 @@ impl<T> Vector<T, 2> {
   where
     T: Extended + Signed,
   {
-    ccw_cmp_around_with(&z.0, &self.0, &p.0, &q.0)
+    Orientation::ccw_cmp_around_with(z, &self.0, &p.0, &q.0)
   }
 
   // FIXME: rename to sort_around_origin
@@ -122,9 +122,23 @@ impl<T> Vector<T, 2> {
     T: Extended + Signed,
   {
     let origin = [T::zero(), T::zero()];
-    pts.sort_unstable_by(|a, b| ccw_cmp_around_with(&[T::one(), T::zero()], &origin, &a.0, &b.0))
+    pts.sort_unstable_by(|a, b| {
+      Orientation::ccw_cmp_around_with(&Vector([T::one(), T::zero()]), &origin, &a.0, &b.0)
+    })
     // L.sortBy (ccwCmpAround c <> cmpByDistanceTo c)
   }
+
+  pub fn cmp_along(&self, p: &Point<T, 2>, q: &Point<T, 2>) -> Ordering
+  where
+    T: crate::PolygonScalar,
+  {
+    // Rotate the vector 90 degrees counterclockwise.
+    match Orientation::along_perp_vector(&p.array, self, &q.array) {
+      Orientation::CounterClockWise => Ordering::Greater,
+      Orientation::ClockWise => Ordering::Less,
+      Orientation::CoLinear => Ordering::Equal,
+    }
+  }
 }
 
 mod add;
@@ -171,3 +185,37 @@ where
     Vector(array_init(|i| self.0.index(i).clone().neg()))
   }
 }
+
+#[cfg(test)]
+mod tests {
+  use super::*;
+  use crate::testing::*;
+
+  use proptest::prelude::*;
+  use test_strategy::proptest;
+
+  #[test]
+  fn unit_1() {
+    let v = Vector([72i8, -113]);
+    let p1 = Point { array: [-84, 93] };
+    let p2 = Point { array: [114, -71] };
+    v.cmp_along(&p1, &p2);
+  }
+
+  #[proptest]
+  fn cmp_along_fuzz(v: Vector<i8, 2>, p1: Point<i8, 2>, p2: Point<i8, 2>) {
+    v.cmp_along(&p1, &p2);
+  }
+
+  #[proptest]
+  fn cmp_along_prop_x(p1: Point<i8, 2>, p2: Point<i8, 2>) {
+    let v = Vector([1, 0]);
+    prop_assert_eq!(v.cmp_along(&p1, &p2), p1.x_coord().cmp(p2.x_coord()));
+  }
+
+  #[proptest]
+  fn cmp_along_prop_y(p1: Point<i8, 2>, p2: Point<i8, 2>) {
+    let v = Vector([0, 1]);
+    prop_assert_eq!(v.cmp_along(&p1, &p2), p1.y_coord().cmp(p2.y_coord()));
+  }
+}
