@@ -3,7 +3,7 @@
 // #![allow(incomplete_features)]
 // #![feature(const_generics)]
 // #![feature(const_evaluatable_checked)]
-#![doc(html_playground_url = "https://rgeometry.org/rgeometry-playground/")]
+#![doc(html_playground_url = "https://rgeometry.org/playground.html")]
 #![doc(test(no_crate_inject))]
 use num_traits::*;
 use std::cmp::Ordering;
@@ -11,14 +11,14 @@ use std::iter::Sum;
 use std::ops::*;
 
 pub mod algorithms;
-mod array;
 pub mod data;
 mod intersection;
 mod matrix;
+mod orientation;
 mod transformation;
 mod utils;
 
-pub use array::Orientation;
+pub use orientation::Orientation;
 
 pub use intersection::Intersects;
 
@@ -90,6 +90,7 @@ pub trait Extended: NumOps<Self, Self> + Ord + Clone {
   fn truncate_signed(val: Self::ExtendedSigned) -> Self;
   fn cmp_slope(p: &[Self; 2], q: &[Self; 2], r: &[Self; 2]) -> std::cmp::Ordering;
   fn cmp_vector_slope(p: &[Self; 2], q: &[Self; 2], r: &[Self; 2]) -> std::cmp::Ordering;
+  fn cmp_perp_vector_slope(p: &[Self; 2], q: &[Self; 2], r: &[Self; 2]) -> std::cmp::Ordering;
 }
 
 macro_rules! fixed_precision {
@@ -103,6 +104,9 @@ macro_rules! fixed_precision {
         val as Self
       }
 
+      // FIXME: These functions should be defined in terms of 'extend_signed' and
+      // 'extend_unsigned'.
+
       fn cmp_slope(p: &[Self; 2], q: &[Self; 2], r: &[Self; 2]) -> std::cmp::Ordering {
         // Return the absolute difference along with its sign.
         // diff(0, 10) => (10, true)
@@ -130,7 +134,7 @@ macro_rules! fixed_precision {
         }
       }
 
-      fn cmp_vector_slope(p: &[Self; 2], q: &[Self; 2], r: &[Self; 2]) -> std::cmp::Ordering {
+      fn cmp_vector_slope(vector: &[Self; 2], p: &[Self; 2], q: &[Self; 2]) -> std::cmp::Ordering {
         // Return the absolute difference along with its sign.
         // diff(0, 10) => (10, true)
         // diff(10, 0) => (10, false)
@@ -143,15 +147,50 @@ macro_rules! fixed_precision {
             (a.wrapping_sub(b) as $uty as $ulong, false)
           }
         }
-        let (ux, ux_neg) = (q[0].unsigned_abs() as $ulong, q[0] < 0);
-        let (vy, vy_neg) = diff(r[1], p[1]);
+        let (ux, ux_neg) = (vector[0].unsigned_abs() as $ulong, vector[0] < 0);
+        let (vy, vy_neg) = diff(q[1], p[1]);
         // neg xor neg = pos = 0
         // neg xor pos = neg = 1
         // pos xor neg = neg = 1
         // pos xor pos = pos = 0
         let ux_vy_neg = ux_neg.bitxor(vy_neg) && ux != 0 && vy != 0;
-        let (uy, uy_neg) = (q[1].unsigned_abs() as $ulong, q[1] < 0);
-        let (vx, vx_neg) = diff(r[0], p[0]);
+        let (uy, uy_neg) = (vector[1].unsigned_abs() as $ulong, vector[1] < 0);
+        let (vx, vx_neg) = diff(q[0], p[0]);
+        let uy_vx_neg = uy_neg.bitxor(vx_neg) && uy != 0 && vx != 0;
+        match (ux_vy_neg, uy_vx_neg) {
+          (true, false) => Ordering::Less,
+          (false, true) => Ordering::Greater,
+          (true, true) => (uy * vx).cmp(&(ux * vy)),
+          (false, false) => (ux * vy).cmp(&(uy * vx)),
+        }
+      }
+
+      fn cmp_perp_vector_slope(
+        vector: &[Self; 2],
+        p: &[Self; 2],
+        q: &[Self; 2],
+      ) -> std::cmp::Ordering {
+        // Return the absolute difference along with its sign.
+        // diff(0, 10) => (10, true)
+        // diff(10, 0) => (10, false)
+        // diff(i8::MIN,i8:MAX) => (255_u16, true)
+        // diff(a,b) = (c, sign) where a = if sign { b-c } else { b+c }
+        fn diff(a: $ty, b: $ty) -> ($ulong, bool) {
+          if b > a {
+            (b.wrapping_sub(a) as $uty as $ulong, true)
+          } else {
+            (a.wrapping_sub(b) as $uty as $ulong, false)
+          }
+        }
+        let (ux, ux_neg) = (vector[1].unsigned_abs() as $ulong, vector[1] > 0);
+        let (vy, vy_neg) = diff(q[1], p[1]);
+        // neg xor neg = pos = 0
+        // neg xor pos = neg = 1
+        // pos xor neg = neg = 1
+        // pos xor pos = pos = 0
+        let ux_vy_neg = ux_neg.bitxor(vy_neg) && ux != 0 && vy != 0;
+        let (uy, uy_neg) = (vector[0].unsigned_abs() as $ulong, vector[0] < 0);
+        let (vx, vx_neg) = diff(q[0], p[0]);
         let uy_vx_neg = uy_neg.bitxor(vx_neg) && uy != 0 && vx != 0;
         match (ux_vy_neg, uy_vx_neg) {
           (true, false) => Ordering::Less,
@@ -180,11 +219,18 @@ macro_rules! arbitrary_precision {
         let slope2 = (q[1].clone() - p[1].clone()) * (r[0].clone() - q[0].clone());
         slope1.cmp(&slope2)
       }
-      fn cmp_vector_slope(p: &[Self; 2], q: &[Self; 2], r: &[Self; 2]) -> std::cmp::Ordering {
+      fn cmp_vector_slope(vector: &[Self;2], p: &[Self; 2], q: &[Self; 2]) -> std::cmp::Ordering {
         Extended::cmp_slope(
           p,
-          &[&p[0] + &q[0], &p[1] + &q[1]],
-          r
+          &[&p[0] + &vector[0], &p[1] + &vector[1]],
+          q
+        )
+      }
+      fn cmp_perp_vector_slope(vector: &[Self;2], p: &[Self; 2], q: &[Self; 2]) -> std::cmp::Ordering {
+        Extended::cmp_slope(
+          p,
+          &[&p[0] - &vector[1], &p[1] + &vector[0]],
+          q
         )
       }
     })*
